<?php

namespace Teknisa\CMP\Modules\Cadastros\Services;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use \Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use \Zeedhi\Framework\DB\StoredProcedure\Param;

class Cmp04700FornecedorAtendeNaoAtendeService
{
    protected $connection;
    
    public function __construct($entityManager, $environment){
        $this->connection = $entityManager->getConnection();
        $this->environment = $environment;
    }
    
    /********************
     *      INSERTS     *
     ********************/
    
    public function insertFornecedor ($row)
    {
        $param = array();
        $param['CDFORNECED'] = $row['CDFORNECED'];
        $param['CDFILIAL']   = $row['CDFILIAL'];
        $param['NRORG']      = $row['NRORG'];
        
        $query = "INSERT INTO REFORFIL (CDFORNECED, CDFILIAL, NRORG)
                    VALUES (:CDFORNECED, :CDFILIAL, :NRORG)";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e) {
            throw $e;
        }
    }
    
    public function insertLista ($nrlista, $cdlista, $nrorg)
    {
        $param = array();
        $param['NRLISTA'] = $nrlista;
        $param['CDLISTA'] = $cdlista;
        $param['NRORG']   = $nrorg;
        
        $query = "INSERT INTO LISTA (NRLISTA, CDLISTA, NRORG) 
                    VALUES (:NRLISTA, :CDLISTA, :NRORG)";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e) {
            throw $e;
        }
    }
    
    public function insertListaTodos ($nrlista)
    {
        $param = array();
        $param['NRLISTA'] = $nrlista;
        
        $query = "INSERT INTO LISTA (NRLISTA, CDLISTA, NRORG) 
                    (SELECT 
                        :NRLISTA, 
                        CDFORNECED, 
                        NRORG 
                    FROM 
                        FORNECEDOR)";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e) {
            throw $e;
        }
    }
    
    public function insertListaTodosFili ($nrlista)
    {
        $param = array();
        $param['NRLISTA'] = $nrlista;
        
        $query = "INSERT INTO LISTA (NRLISTA, CDLISTA, NRORG) 
                    (SELECT 
                        :NRLISTA, 
                        CDFILIAL, 
                        NRORG 
                    FROM 
                        FILIAL)";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e) {
            throw $e;
        }
    }
    
    public function insertListaTodosFiliReg ($nrlista, $cdRegiao)
    {
        $param = array();
        $param['NRLISTA']    = $nrlista;
        $param['P_CDREGIAO'] = $cdRegiao;
        
        $query = "  INSERT INTO LISTA (NRLISTA, CDLISTA, NRORG)
                    (SELECT 
                        :NRLISTA, 
                        CDFILIAL, 
                        NRORG
                    FROM 
                        FILIAL
                    WHERE 
                        INSTR(:P_CDREGIAO, CDREGIAO) > 0)";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e) {
            throw $e;
        }
    }
    
    public function insertCodPeri ($row)
    {
        $param = array();
        $param['CDFORNECED'] = $row['CDFORNECED'];
        $param['CDFILIAL'] = $row['CDFILIAL'];
        $param['CDPERIENTR'] = $row['CDPERIENTR'];
        $param['NRORG'] = $row['NRORG'];
        
        $query = "  INSERT INTO REFORFIL
                        (CDFORNECED, CDFILIAL, CDPERIENTR, NRORG)
                    VALUES
                        (:CDFORNECED, :CDFILIAL, :CDPERIENTR, :NRORG)";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e) {
            throw $e;
        }
    }
    
    public function insertReforfil ($nrlistaForn, $nrlistaFili)
    {
        $param = array();
        $param['P_NRLISTAFORN'] = $nrlistaForn;
        $param['P_NRLISTAFIL']  = $nrlistaFili;
        
        $query = "  INSERT INTO REFORFIL(CDFORNECED, CDFILIAL, NRORG)
                    SELECT
                        FORNECEDOR.CDFORNECED,
                        FILIAL.CDFILIAL,
                        FILIAL.NRORG
                    FROM
                        FORNECEDOR,
                        FILIAL,
                        LISTA,
                        LISTA L
                    WHERE
                        LISTA.NRLISTA = :P_NRLISTAFORN AND
                        LISTA.CDLISTA = FORNECEDOR.CDFORNECED  AND
                        L.NRLISTA = :P_NRLISTAFIL AND
                        L.CDLISTA = FILIAL.CDFILIAL AND
                    NOT EXISTS
                        (SELECT 
                            CDFILIAL
                        FROM 
                            REFORFIL
                        WHERE 
                            FORNECEDOR.CDFORNECED = REFORFIL.CDFORNECED AND
                            FILIAL.CDFILIAL = REFORFIL.CDFILIAL)";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e) {
            throw $e;
        }
    }
    
    public function insertReforfilAtende ($nrlistaForn, $nrlistaFili, $cdPerientr)
    {
        $param = array();
        $param['P_NRLISTAFORN'] = $nrlistaForn;
        $param['P_NRLISTAFIL']  = $nrlistaFili;
        $param['P_CDPERIENTR']  = $cdPerientr;
        
        $query = "  INSERT INTO REFORFIL(CDFORNECED, CDFILIAL, CDPERIENTR, NRORG)
                    SELECT
                        FORNECEDOR.CDFORNECED,
                        FILIAL.CDFILIAL,
                        PERIENTR.CDPERIENTR,
                        FORNECEDOR.NRORG
                    FROM
                        FORNECEDOR,
                        FILIAL,
                        PERIENTR,
                        LISTA,
                        LISTA L
                    WHERE
                        LISTA.NRLISTA = :P_NRLISTAFORN AND
                        LISTA.CDLISTA = FORNECEDOR.CDFORNECED AND
                        L.NRLISTA = :P_NRLISTAFIL AND
                        L.CDLISTA = FILIAL.CDFILIAL AND
                        (PERIENTR.CDPERIENTR = :P_CDPERIENTR) AND
                    NOT EXISTS
                        (SELECT 
                            CDFILIAL
                        FROM 
                            REFORFIL
                        WHERE
                            FORNECEDOR.CDFORNECED = REFORFIL.CDFORNECED AND
                            FILIAL.CDFILIAL = REFORFIL.CDFILIAL)";
                            
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e) {
            throw $e;
        }
    }
    
    
    /********************
     *      UPDATES     *
     ********************/
     
     
    public function updateCodPeri ($row){
        $param = array();
        $param['CDFORNECED']     = $row['CDFORNECED'];
        $param['CDFILIAL']       = $row['CDFILIAL'];
        $param['CDPERIENTR']     = $row['CDPERIENTR'];
        $param['OLD_CDFORNECED'] = $row['CDFORNECED'];
        $param['OLD_CDFILIAL']   = $row['CDFILIAL'];
        
        $query = "  UPDATE 
                        REFORFIL 
                    SET 
                        CDFORNECED = :CDFORNECED, 
                        CDFILIAL = :CDFILIAL, 
                        CDPERIENTR = :CDPERIENTR 
                    WHERE 
                        CDFORNECED = :OLD_CDFORNECED AND 
                        CDFILIAL = :OLD_CDFILIAL";
                        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e) {
            throw $e;
        }
    }
    
    public function updatePrCalendar ($row)
    {
        $param = array();
        $param['CDPERIENTR']             = $row['CDPERIENTR'];
        $param['IDUTCALENDCMPGERACAOAF'] = $row['IDUTCALENDCMPGERACAOAF'];
        $param['OLD_CDFORNECED']         = $row['CDFORNECED'];
        $param['OLD_CDFILIAL']           = $row['CDFILIAL'];
        
        $query = "  UPDATE 
                        REFORFIL
                    SET
                        CDPERIENTR = :CDPERIENTR,
                        IDUTCALENDCMPGERACAOAF = :IDUTCALENDCMPGERACAOAF
                    WHERE
                        CDFORNECED = :OLD_CDFORNECED AND
                        CDFILIAL = :OLD_CDFILIAL";
                        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e) {
            throw $e;
        }
    }
    
    public function updateReforfil ($nrlistaForn, $nrlistaFili)
    {
        $param = array();
        $param['P_NRLISTAFORN'] = $nrlistaForn;
        $param['P_NRLISTAFIL']  = $nrlistaFili;
        
        $query = "  UPDATE 
                        REFORFIL
                    SET 
                        CDPERIENTR = NULL
                    WHERE 
                        EXISTS(
                            SELECT 
                                1
                            FROM 
                                LISTA
                            WHERE 
                                LISTA.NRLISTA = :P_NRLISTAFORN AND
                                LISTA.CDLISTA = CDFORNECED) AND
                                EXISTS(
                                    SELECT 
                                        1
                                    FROM 
                                        LISTA L
                                    WHERE 
                                        L.NRLISTA = :P_NRLISTAFIL AND
                                        L.CDLISTA = CDFILIAL)";
                                        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e) {
            throw $e;
        }
    }
    
    public function updateReforfilAtende ($nrlistaForn, $nrlistaFili, $cdPerientr)
    {
        $param = array();
        $param['P_NRLISTAFORN'] = $nrlistaForn;
        $param['P_NRLISTAFIL']  = $nrlistaFili;
        $param['P_CDPERIENTR']  = $cdPerientr;
        
        $query = "  UPDATE 
                        REFORFIL
                    SET 
                        CDPERIENTR = :P_CDPERIENTR
                    WHERE 
                        EXISTS(
                            SELECT
                                1
                            FROM
                                LISTA
                            WHERE
                                LISTA.NRLISTA = :P_NRLISTAFORN AND
                                LISTA.CDLISTA = CDFORNECED) AND
                                EXISTS(
                                    SELECT 
                                        1
                                    FROM 
                                        LISTA L
                                    WHERE 
                                        L.NRLISTA = :P_NRLISTAFIL AND
                                        L.CDLISTA = CDFILIAL)";
                                        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e) {
            throw $e;
        }
    }
    
    
    /********************
     *      DELETES     *
     ********************/
     
     
    public function deleteFornecedor ($row)
    {
        $param = array();
        $param['OLD_CDFORNECED'] = $row['CDFORNECED'];
        $param['OLD_CDFILIAL'] = $row['CDFILIAL'];
        
        $query = "  DELETE FROM 
                        REFORFIL
                    WHERE
                        CDFORNECED = :OLD_CDFORNECED AND
                        CDFILIAL = :OLD_CDFILIAL";
                        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e) {
            throw $e;
        }
    }
    
    public function deleteFornecedorAtendeIns ($row)
    {
        $param = array();
        $param['CDFORNECED'] = $row['CDFORNECED'];
        $param['CDFILIAL']   = $row['CDFILIAL'];
        $param['NRORG']      = $row['NRORG'];
        
        $query = "  INSERT INTO REFORFIL (CDFORNECED, CDFILIAL, NRORG)
                    VALUES (:CDFORNECED, :CDFILIAL, :NRORG)";
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e) {
            throw $e;
        }
    }
    
    public function deleteFornecedorAtendeUpd ($row)
    {
        $param = array();
        $param['CDPERIENTR']             = null;
        $param['IDUTCALENDCMPGERACAOAF'] = null;
        $param['OLD_CDFORNECED']         = $row['CDFORNECED'];
        $param['OLD_CDFILIAL']           = $row['CDFILIAL'];
        
        $query = "  UPDATE
                        REFORFIL
                    SET
                        CDPERIENTR = :CDPERIENTR,
                        IDUTCALENDCMPGERACAOAF = :IDUTCALENDCMPGERACAOAF
                    WHERE
                        CDFORNECED = :OLD_CDFORNECED AND
                        CDFILIAL = :OLD_CDFILIAL";
                        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e) {
            throw $e;
        }
    }
    
    public function deleteLista ($nrlista)
    {
        $param = array();
        $param['P_NRLISTA'] = $nrlista;
        
        $query = "  DELETE FROM
                       LISTA
                    WHERE
                       NRLISTA = :P_NRLISTA";
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e) {
            throw $e;
        }
    }
    
    public function deleteReforfil ($nrlistaForn, $nrlistaFili)
    {
        $param = array();
        $param['P_NRLISTAFORN'] = $nrlistaForn;
        $param['P_NRLISTAFIL'] = $nrlistaFili;
        
        $query = "  DELETE FROM 
                        REFORFIL
                    WHERE
                        EXISTS(
                            SELECT 
                                1
                            FROM 
                                LISTA
                            WHERE 
                                LISTA.NRLISTA = :P_NRLISTAFORN AND
                                LISTA.CDLISTA = CDFORNECED) AND
                                EXISTS(
                                    SELECT 
                                        1
                                    FROM 
                                        LISTA L
                                    WHERE 
                                        L.NRLISTA = :P_NRLISTAFIL AND
                                        L.CDLISTA = CDFILIAL)";
                                        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e) {
            throw $e;
        }
    }
    
    
    /********************
     *      SELECTS     *
     ********************/
     
     
    public function findFornecedor($row) 
    {
        $param = array();
        $param['P_CDFORNECED'] = $row['CDFORNECED'];
        $param['P_CDFILIAL']   = $row['CDFILIAL'];
        $param['P_NRORG']        = $row['NRORG'];
        
        $query = "  SELECT
                        CDFILIAL,
                        CDFORNECED,
                        NRORG
                    FROM
                        REFORFIL
                    WHERE
                        (CDFORNECED = :P_CDFORNECED) AND
                        (CDFILIAL   = :P_CDFILIAL) AND
                        (NRORG      = :P_NRORG)";
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch (Exception $e) {
            throw $e;
        }
    }
    
    public function verifyFornecedor($row)
    {
        $param = array();
        $param['P_CDFORNECED'] = $row['CDFORNECED'];
        $param['P_CDFILIAL']   = $row['CDFILIAL'];
        $param['P_NRORG']      = $row['NRORG'];
        
        $query = "  SELECT
                        CDFORNECED
                    FROM
                        FILIAL,
                        REFOREG
                    WHERE
                        REFOREG.CDFORNECED = :P_CDFORNECED AND
                        FILIAL.CDFILIAL = :P_CDFILIAL AND
                        FILIAL.CDREGIAO = REFOREG.CDREGIAO AND
                        FILIAL.NRORG = REFOREG.NRORG AND
                        REFOREG.NRORG = :P_NRORG";
                        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch (Exception $e) {
            throw $e;
        }
    }
    
    public function verifyFornReforfil($row)
    {
        $param = array();
        $param['CDFORNECED'] = $row['CDFORNECED'];
        $param['CDFILIAL']   = $row['CDFILIAL'];
        $param['NRORG']      = $row['NRORG'];
        
        $query = "  SELECT
                        CDFORNECED,
                        CDFILIAL
                    FROM
                        REFORFIL
                    WHERE
                        CDFORNECED = :CDFORNECED AND
                        CDFILIAL = :CDFILIAL AND
                        NRORG = :NRORG";
                        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch (Exception $e) {
            throw $e;
        }
    }
    
    public function verifyCdPeriod($row)
    {
        $param = array();
        $param['P_CDPERIENTR'] = $row['CDPERIENTR'];
        $param['P_NRORG']      = $row['NRORG'];
        
        $query = "  SELECT
                        CDPERIENTR,
                        NMPERIENTR,
                        IDTIPODIA
                    FROM
                        PERIENTR
                    WHERE
                        CDPERIENTR = :P_CDPERIENTR AND
                        NRORG = :P_NRORG";
                        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch (Exception $e) {
            throw $e;
        }
    }
    
    public function findCodPeri($row)
    {
        $param = array();
        $param['P_CDFILIAL']   = $row['CDFILIAL'];
        $param['P_NRORG']      = $row['NRORG'];
        
        $query = "  SELECT
                        REFORFIL.CDFORNECED,
                        REFORFIL.CDFILIAL,
                        NVL(REFORFIL.IDUTCALENDCMPGERACAOAF,'N') IDUTCALENDCMPGERACAOAF,
                        FILIAL.NMFILIAL,
                        FORNECEDOR.IDTPIJURFORN,
                        FORNECEDOR.NRINSJURFORN,
                        FORNECEDOR.NMRAZSOCFORN,
                        FORNECEDOR.NMFANTFORN,
                        PERIENTR.CDPERIENTR,
                        PERIENTR.NMPERIENTR,
                        FILIAL.NRORG
                    FROM
                        REFORFIL,
                        FILIAL,
                        FORNECEDOR,
                        PERIENTR
                    WHERE
                        REFORFIL.CDFILIAL =:P_CDFILIAL AND
                        REFORFIL.CDFORNECED = FORNECEDOR.CDFORNECED AND
                        REFORFIL.CDFILIAL = FILIAL.CDFILIAL AND
                        REFORFIL.CDPERIENTR = PERIENTR.CDPERIENTR AND
                        REFORFIL.CDPERIENTR IS NOT NULL AND
                        REFORFIL.NRORG = FILIAL.NRORG AND
                        FILIAL.NRORG = FORNECEDOR.NRORG AND
                        FORNECEDOR.NRORG = PERIENTR.NRORG AND
                        PERIENTR.NRORG =:P_NRORG
                        
                    UNION
                    
                    SELECT
                        FORNECEDOR.CDFORNECED,
                        FILIAL.CDFILIAL,
                        '' AS IDUTCALENDCMPGERACAOAF,
                        FILIAL.NMFILIAL,
                        FORNECEDOR.IDTPIJURFORN,
                        FORNECEDOR.NRINSJURFORN,
                        FORNECEDOR.NMRAZSOCFORN,
                        FORNECEDOR.NMFANTFORN,
                        NULL AS CDPERIENTR,
                        NULL AS NMPERIENTR,
                        FILIAL.NRORG
                    FROM
                        FILIAL,
                        FORNECEDOR
                    WHERE
                        FILIAL.CDFILIAL =:P_CDFILIAL AND
                        NOT EXISTS (
                            SELECT
                                CDFILIAL
                            FROM
                                REFORFIL R
                            WHERE
                                R.CDFILIAL = FILIAL.CDFILIAL AND
                                R.CDFORNECED = FORNECEDOR.CDFORNECED
                        )
                    ORDER BY
                        4,5";
                        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch (Exception $e) {
            throw $e;
        }
    }
    
    public function validaReforeg($nrlistaForn, $nrlistaFili)
    {
        $param = array();
        $param['P_NRLISTAFORN'] = $nrlistaForn;
        $param['P_NRLISTAFIL']  = $nrlistaFili;
        
        $query = "  SELECT
                        FILIAL.CDFILIAL||' - '|| FILIAL.NMFILIAL AS FILIAL,
                        FILIAL.CDREGIAO||' - '|| REGIAO.NMREGIAO AS REGIAO,
                        FORNECEDOR.NRINSJURFORN FORNECEDOR,
                        FORNECEDOR.NMFANTFORN,
                        FORNECEDOR.IDTPIJURFORN,
                        FORNECEDOR.NRORG
                    FROM
                        FORNECEDOR,
                        REGIAO,
                        FILIAL,
                        LISTA,
                        LISTA L
                    WHERE
                        LISTA.NRLISTA = :P_NRLISTAFORN AND
                        LISTA.CDLISTA = FORNECEDOR.CDFORNECED AND
                        L.NRLISTA = :P_NRLISTAFIL AND
                        L.CDLISTA = FILIAL.CDFILIAL AND
                        (REGIAO.CDREGIAO = FILIAL.CDREGIAO) AND
                        ((EXISTS(
                            SELECT
                                CDFORNECED
                            FROM
                                REFOREG
                            WHERE
                                REFOREG.CDFORNECED = FORNECEDOR.CDFORNECED AND
                                REFOREG.CDREGIAO   = FILIAL.CDREGIAO)))
                    ORDER BY
                        FORNECEDOR.NRINSJURFORN,
                        FILIAL.CDREGIAO,
                        FILIAL.CDFILIAL";
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch (Exception $e) {
            throw $e;
        }
    }
    
    public function validaReforfil($nrlistaForn, $nrlistaFili)
    {
        $param = array();
        $param['P_NRLISTAFORN'] = $nrlistaForn;
        $param['P_NRLISTAFIL']  = $nrlistaFili;
        
        $query = "  SELECT
                        CDFORNECED
                    FROM
                        REFORFIL,
                        LISTA,
                        LISTA L
                    WHERE
                        LISTA.NRLISTA = :P_NRLISTAFORN AND
                        LISTA.CDLISTA = CDFORNECED AND
                        L.NRLISTA = :P_NRLISTAFIL AND
                        L.CDLISTA = CDFILIAL AND
                        (REFORFIL.CDPERIENTR IS NOT NULL)";
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch (Exception $e) {
            throw $e;
        }
    }
    
    
    /***********************
     *      PROCEDURES     *
     * *********************/
     
     
     public function procExportFornecedor($P_NRLISTA, $P_CDFILORIG)
     {
        $connection = $this->connection;
        $procedure = new StoredProcedure($connection, 'EXPORTA_REFORFIL');
        $procedure->addParam(new Param('P_NRLISTA', Param::PARAM_INPUT, $P_NRLISTA, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILORIG', Param::PARAM_INPUT, $P_CDFILORIG, Param::PARAM_TYPE_STR));
        $rowsaffect = $procedure->execute();
    }
}