<?php
namespace Teknisa\CMP\Services; 

use \Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use \Zeedhi\Framework\DB\StoredProcedure\Param;
use \Teknisa\Model\Suprimentos\Entities\Autoforne;
use \Teknisa\Model\Suprimentos\Entities\Itemautf;

class FechamentoAFService {
    
    protected $entityManager;
    protected $connection;
    protected $environment;

    public function __construct($entityManager, $environment) {
        $this->entityManager = $entityManager;
        $this->connection = $entityManager->getConnection();
        $this->environment = $environment;
    }
    
    
    public function executaSql($sql, $params, $consulta = true) {
        try{
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            
            if($consulta == true) {
                return $std->fetchAll();
            }
            
            return true;
        } catch (\Exception $e){
            throw $e;
        } 
    }
    
    
    public function verificaExisteAF($cdfildest, $cdfillanc, $nrautoforn) {
        
        $sql = "SELECT AFORIG.CDFORNECED AS CDFORNECEDORIG, AFDEST.CDFORNECED AS CDFORNECEDDEST, TO_CHAR(AFDEST.DTAUTOFORN, 'DD/MM/YYYY') AS DTAUTOFORN, AFORIG.CDOPERADOR FROM AUTOFORNE AFORIG, AUTOFORNE AFDEST WHERE AFORIG.CDFILDEST = :CDFILDEST AND AFORIG.CDFILLANC = :CDFILLANC AND AFORIG.NRAUTOFORN = :NRAUTOFORN AND AFORIG.NRORG = :NRORG AND AFDEST.CDFILDEST = :CDFILDEST AND AFDEST.CDFILLANC = :CDFILLANC AND AFDEST.NRAUTOFORN = :NRAUTOFORNDEST AND AFDEST.NRORG = :NRORG";
        
        $params = [
            "CDFILDEST" => $cdfildest,
            "CDFILLANC" => $cdfillanc,
            "NRAUTOFORN" => $nrautoforn,
            "NRAUTOFORNDEST" => $nrautoforn,
            "NRORG" => $this->environment->getNrOrgTrab()
        ];
        
        $existeAF = self::executaSql($sql, $params);
        
        return count($existeAF) ? $existeAF[0] : false;
        
    }
    
    
    public function retornaForneExtra($cdfildest, $cdfillanc, $nrautoforn) {
        
        $sql = "SELECT AUTOFORNE.IDAUTOFORNEX,(SELECT 1 FROM FORNECMPEXTRA WHERE FORNECMPEXTRA.CDFORNECED = AUTOFORNE.CDFORNECED AND FORNECMPEXTRA.CDPAIS = ENDEFILI.CDPAIS AND FORNECMPEXTRA.SGESTADO = ENDEFILI.SGESTADO AND FORNECMPEXTRA.CDMUNICIPIO = ENDEFILI.CDMUNICIPIO AND FORNECMPEXTRA.NRORG = AUTOFORNE.NRORG) AUXFORNEXTRA FROM AUTOFORNE, ENDEFILI WHERE AUTOFORNE.CDFILDEST = :CDFILDEST AND AUTOFORNE.CDFILLANC = :CDFILLANC AND AUTOFORNE.NRAUTOFORN = :NRAUTOFORN AND AUTOFORNE.NRORG = :NRORG AND ENDEFILI.CDFILIAL = :CDFILDEST AND ENDEFILI.IDTPENDEFILI = 'P' AND ENDEFILI.NRORG = :NRORG";
        
        $params = [
            "CDFILDEST" => $cdfildest,
            "CDFILLANC" => $cdfillanc,
            "NRAUTOFORN" => $nrautoforn,
            "NRORG" => $this->environment->getNrOrgTrab()
        ];
        
        $fornExtra = self::executaSql($sql, $params);
        
        return count($fornExtra) ? $fornExtra[0] : false;
        
    }
    
    
    public function verificaInconsPreco($cdfildest) {
        
        $sql = "SELECT NVL(PARASUPRI.IDAFCOMPLIMGER, 'N') IDAFCOMPLIMGER, IDINCONSPRECSUP FROM PARASUPRI, FILIAL WHERE FILIAL.CDFILIAL = :CDFILDEST AND PARASUPRI.NRORG = :NRORG AND PARASUPRI.CDFILIAL = NVL(FILIAL.CDFILCENTCMP, FILIAL.CDFILIAL) AND PARASUPRI.NRORG = FILIAL.NRORG";
        
        $params = [
            "CDFILDEST" => $cdfildest,
            "NRORG" => $this->environment->getNrOrgTrab()
        ];
        
        $incons = self::executaSql($sql, $params);
        
        return count($incons) ? $incons[0] : false;
        
    }
    
    
    public function retornaPrecoItensAF($cdfildest, $cdfillanc, $nrautoforn) {
        
        $sql = "SELECT ITEMAUTF.CDPRODUTO, PRODUTO.VRFATOCONV, TO_CHAR(ITEMAUTF.DTPROGENTAF, 'DD/MM/YYYY') AS DTPROGENTAF, ITEMAUTF.CDMARCA, ITEMAUTF.QTPROGENTAF, PRODUTO.SGUNIDADE, ITEMAUTF.VRPREUNIAF,(SELECT MIN(C.VRBRUCMPPROD) AS VRUNITITEM FROM CUSTOCMPPROD C, ITEMAUTF I, FILIAL F, FILIAL FC WHERE I.CDFILDEST = :CDFILDEST AND I.CDFILLANC = :CDFILLANC AND I.NRAUTOFORN = :NRAUTOFORN AND I.DTPROGENTAF = ITEMAUTF.DTPROGENTAF AND I.CDMARCA = ITEMAUTF.CDMARCA AND I.CDPRODUTO = ITEMAUTF.CDPRODUTO AND I.NRORG = ITEMAUTF.NRORG AND C.CDPRODUTO = PRODUTO.CDPRODESTO AND C.NRORG = PRODUTO.NRORG AND I.CDFILDEST = F.CDFILIAL AND I.NRORG = F.NRORG AND C.CDFILIAL = FC.CDFILIAL AND C.NRORG = FC.NRORG AND F.CDREGIAO = FC.CDREGIAO AND I.VRPREUNIAF > 0 AND C.VRBRUCMPPROD > 0 AND C.VRBRUCMPPROD < (I.VRPREUNIAF/PRODUTO.VRFATOCONV) AND DTCUSTCMPPRO = (SELECT MAX(D.DTCUSTCMPPRO) FROM FILIAL FIL, CUSTOCMPPROD D WHERE D.CDPRODUTO = PRODUTO.CDPRODESTO AND D.NRORG = PRODUTO.NRORG AND D.VRBRUCMPPROD > 0 AND D.CDFILIAL = FIL.CDFILIAL AND D.NRORG = FIL.NRORG AND FIL.CDREGIAO = FC.CDREGIAO AND D.DTCUSTCMPPRO <= I.DTPROGENTAF)) VRUNITITEM FROM ITEMAUTF, PRODUTO WHERE ITEMAUTF.CDPRODUTO = PRODUTO.CDPRODUTO AND ITEMAUTF.NRORG = PRODUTO.NRORG AND ITEMAUTF.CDFILDEST = :CDFILDEST AND ITEMAUTF.CDFILLANC = :CDFILLANC AND ITEMAUTF.NRAUTOFORN = :NRAUTOFORN AND ITEMAUTF.NRORG = :NRORG";
        
        $params = [
            "CDFILDEST" => $cdfildest,
            "CDFILLANC" => $cdfillanc,
            "NRAUTOFORN" => $nrautoforn,
            "NRORG" => $this->environment->getNrOrgTrab()
        ];
        
        $itensAF = self::executaSql($sql, $params);
        
        return count($itensAF) ? $itensAF : false;
        
    }
    
    
    public function atualizaInconsAF($cdfildest, $cdfillanc, $nrautoforn, $idsituatuaaf, $dsinconsaf = null) {

        $autoforne = $this->entityManager->getRepository('\Teknisa\Model\Suprimentos\Entities\Autoforne')
            ->findOneBy(array(
                'cdfildest' => $cdfildest,
                'cdfillanc' => $cdfillanc,
                'nrautoforn' => $nrautoforn,
                'nrorg' => $this->environment->getNrOrgTrab()
            ));

        if($autoforne instanceof Autoforne) {

            if($dsinconsaf) {

                $dsinconsaf = $autoforne->getDsinconsaf() . ' ' . $dsinconsaf;
                $autoforne->setDsinconsaf($dsinconsaf);
                
            }

            $autoforne->setIdsituatuaaf($idsituatuaaf);
            $autoforne->setCdgerente(null);
            
            $autoforne->setDtultatu(new \DateTime());
            $autoforne->setNrorgultatu($this->environment->getNrOrgTrab());
            $autoforne->setCdoperultatu($this->environment->getCdoperador());

            $this->entityManager->persist($autoforne);
            
        }
        
        return true;
        
    }
    
    
    public function atualizaInconsItemAF($cdfildest, $cdfillanc, $nrautoforn, $cdproduto, $cdmarca, $dtprogentaf, $dsinconsitaf) {

        $itemautf = $this->entityManager->getRepository('\Teknisa\Model\Suprimentos\Entities\Itemautf')
            ->findOneBy(array(
                'cdfildest' => $cdfildest,
                'cdfillanc' => $cdfillanc,
                'nrautoforn' => $nrautoforn,
                'cdproduto' => $cdproduto,
                'cdmarca' => $cdmarca,
                'dtprogentaf' => $dtprogentaf,
                'nrorg' => $this->environment->getNrOrgTrab()
            ));

        if($itemautf instanceof Itemautf) {

            $dsinconsitaf = $itemautf->getDsinconsitaf() . ' ' . $dsinconsitaf;
            $itemautf->setDsinconsitaf($dsinconsitaf);

            $itemautf->setDtultatu(new \DateTime());
            $itemautf->setNrorgultatu($this->environment->getNrOrgTrab());
            $itemautf->setCdoperultatu($this->environment->getCdoperador());
            
            $this->entityManager->persist($itemautf);
            
        }
        
        return true;
        
    }
    
    
    public function atualizaTipoContaPagar($cdfildest, $cdfillanc, $nrautoforn) {
        
        $update = "UPDATE AUTOFORNE SET CDTPCTPAGAR =(SELECT PRODUTO.CDTPCTPAGAR FROM PRODUTO, (SELECT MAX(PRODUTO.CDARVPROD) AS CDARVPROD FROM PRODUTO, (SELECT MIN(A.CDARVPROD) AS CDARVPROD FROM (SELECT CDARVPROD, MAX(QTPROGENTAF*VRPREUNIAF) FROM ITEMAUTF, PRODUTO P WHERE P.CDPRODUTO = ITEMAUTF.CDPRODUTO AND ITEMAUTF.CDFILDEST = :CDFILDEST AND ITEMAUTF.CDFILLANC = :CDFILLANC AND ITEMAUTF.NRAUTOFORN = :NRAUTOFORN GROUP BY CDARVPROD HAVING MAX(QTPROGENTAF *VRPREUNIAF) = (SELECT MAX(QTPROGENTAF*VRPREUNIAF) FROM ITEMAUTF WHERE ITEMAUTF.CDFILDEST = :CDFILDEST AND ITEMAUTF.CDFILLANC = :CDFILLANC AND ITEMAUTF.NRAUTOFORN = :NRAUTOFORN) )A ) ITEMAUTF WHERE ITEMAUTF.CDARVPROD LIKE PRODUTO.CDARVPROD || '%' AND PRODUTO.CDTPCTPAGAR IS NOT NULL ) TIPOCTPAGAR WHERE PRODUTO.CDARVPROD = TIPOCTPAGAR.CDARVPROD ), DTULTATU = TO_DATE(:DTULTATU, 'DD/MM/YYYY'), CDOPERULTATU = :CDOPERULTATU, NRORGULTATU = :NRORGULTATU WHERE AUTOFORNE.CDFILDEST = :CDFILDEST AND AUTOFORNE.CDFILLANC = :CDFILLANC AND AUTOFORNE.NRAUTOFORN = :NRAUTOFORN AND AUTOFORNE.NRORG = :NRORG AND AUTOFORNE.CDTPCTPAGAR IS NULL";
        
        $dtatual = new \DateTime();
        
        $params = [
            "CDFILDEST" => $cdfildest,
            "CDFILLANC" => $cdfillanc,
            "NRAUTOFORN" => $nrautoforn,
            "NRORG" => $this->environment->getNrOrgTrab(),
            "DTULTATU" => $dtatual->format('d/m/Y'),
            "CDOPERULTATU" => $this->environment->getCdoperador(),
            "NRORGULTATU" => $this->environment->getNrOrgTrab()
        ];
        
        self::executaSql($update, $params, false);
        
        return true;
        
    }
    
    
    public function proceFechamentoAF($cdfildest, $cdfillanc, $nrautoforn) {
        //without commit in procedure
        $procedure = new StoredProcedure($this->connection, 'FECHAMENTO_AF');
        
        $procedure->addParam(new Param('P_CDFILDEST', Param::PARAM_INPUT, $cdfildest, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILLANC', Param::PARAM_INPUT, $cdfillanc, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRAUTOFORN', Param::PARAM_INPUT, $nrautoforn, Param::PARAM_TYPE_STR));
        
        $procedure->execute();
        
    }
    
    public function getParaSupri($cdfilial)
	{
	    $sql = "SELECT * FROM PARASUPRI WHERE CDFILIAL = :P_CDFILIAL AND NRORG = :NRORG";
        
        $params = [
            "P_CDFILIAL" => $cdfilial,
            "NRORG" => $this->environment->getNrOrg()
        ];
        
        $dataset = self::executaSql($sql, $params);
        
        return count($dataset) ? $dataset[0] : false;
	}
	
	public function updateOrcaFinaAF($cdfildest, $cdfillanc, $nrautoforn)
	{
        $update = "UPDATE AUTOFORNE
                      SET IDORCAFINULTRAP = 'S'
                    WHERE CDFILDEST  = :P_CDFILDEST
                      AND CDFILLANC  = :P_CDFILLANC
                      AND NRAUTOFORN = :P_NRAUTOFORN
                      AND NRORG      = :NRORG";
        
        $params = [
            "P_CDFILDEST"  => $cdfildest,
            "P_CDFILLANC"  => $cdfillanc,
            "P_NRAUTOFORN" => $nrautoforn,
            "NRORG"        => $this->environment->getNrOrg()
        ];
        
        return self::executaSql($update, $params, false);
    }
}
   